@@ -92,7 +92,10 @@ class CacheManager(PickleStore):
         timeout = timeout or self.default_timeout
         if timeout>MAX_EXPIRE_SECONDS:
             timeout = max(0, timeout-int(time.time()))
-        self.setex(self.key_prefix+key, timeout, value)
+        if self._dialect=='Redis':
+            self.setex(self.key_prefix+key, value, timeout)
+        else:
+            self.setex(self.key_prefix+key, timeout, value)
 
     def add(self, key, value, timeout=None):
         """Works like :meth:`set` but does not overwrite the values of already
