@@ -10,9 +10,10 @@ Object Hash Mapper
 
 __all__ = [
     'STATE_NEW', 'STATE_DIRTY', 'STATE_UNCHANGED', 'MAX_EXPIRE_SECONDS',
-    'Attr', 'NumberAttr', 'IntAttr', 'FloatAttr',
-    'TimestampAttr', 'SerialAttr', 'BooleanAttr',
-    'StringAttr', 'UnicodeAttr', 'Hash', 'Manager'
+    'Attr', 'EnumAttr', 'NumberAttr', 'IntAttr', 'FloatAttr',
+    'TimestampAttr', 'SerialAttr', 'BooleanAttr', 'StringAttr',
+    'PeriodSerialAttr', 'UnicodeAttr',
+    'Hash', 'Manager'
 ]
 
 import time
@@ -23,7 +24,7 @@ except ImportError:
 from decimal import Decimal
 from redis import WatchError
 from .stores import Store
-from .serials import SerialsManager, Serial
+from .serials import SerialsManager, Serial, PeriodSerial, PERIOD_DAILY
 
 STATE_NEW, STATE_DIRTY, STATE_UNCHANGED = 'new', 'dirty', 'unchanged'
 
@@ -86,6 +87,24 @@ class Attr(object):
         self.set_value(obj, value)
 
 
+class EnumAttr(Attr):
+    def __init__(self, field=None, default=None, required=False,
+                 values=None):
+        if type(values) not in [tuple, list, set]:
+            raise TypeError('"values" should be tuple, list or set')
+        values = list(values)
+        Attr.__init__(self, field, default, required)
+        if not self.required:
+            values.append(None)
+        if self.default!=None and self.default not in values:
+            raise ValueError('"default" neither in values nor None')
+        self.values = set(values)
+
+    def validate(self, value):
+        if value not in self.values:
+            raise ValueError('"%r" is invalid' % value)
+
+
 class NumberAttr(Attr):
     def __init__(self, field=None, default=Decimal('0'),
                  min_value=None, max_value=None):
@@ -135,13 +154,14 @@ class TimestampAttr(IntAttr):
 
 
 class SerialAttr(IntAttr):
-    def __init__(self, field, default=None, as_key=False):
+    def __init__(self, field=None, default=None, as_key=False):
         IntAttr.__init__(self, field, default, as_key, None, None)
 
     def set_value(self, obj, value):
         if value!=None:
-            value = int(value)
-        Attr.set_value(self, obj, value)
+            IntAttr.set_value(self, obj, value)
+        else:
+            Attr.set_value(self, obj, value)
     
     def dumps(self, obj):
         value = self.get_value(obj)
@@ -260,6 +280,37 @@ class StringAttr(Attr):
         self.set_value(obj, data)
 
 
+class PeriodSerialAttr(StringAttr):
+    def __init__(self, field=None, default=None, as_key=False,
+                 min_length=0, max_length=None, encoding='utf8',
+                 period=PERIOD_DAILY, pattern=u'%(moment)s%(number)08d',
+                 moment_format=None):
+        StringAttr.__init__(self, field, default, as_key, None, None, encoding)
+        self.period = str(period)
+        self.pattern = unicode(pattern)
+        self.moment_format = unicode(moment_format) if moment_format else None
+
+    def set_value(self, obj, value):
+        if value!=None:
+            StringAttr.set_value(self, obj, value)
+        else:
+            Attr.set_value(self, obj, value)
+    
+    def dumps(self, obj):
+        value = self.get_value(obj)
+        if value==None:
+            classname = obj.__class__.__name__
+            name = '%s.%s' % (classname, self.field)
+            serials = obj.manager.serials
+            if not serials.has_serial(name):
+                serial = PeriodSerial(name, self.period, pattern=self.pattern,
+                                      moment_format=self.moment_format)
+                serials.add_serial(serial)
+            value = serials.incr(name)
+            self.set_value(obj, value)
+        return str(value)
+
+
 class UnicodeAttr(StringAttr):
 
     def set_value(self, obj, value):
@@ -390,6 +441,12 @@ class Hash(object):
     @property
     def manager(self):
         return self.__ohm_manager__
+    
+    def save(self, manager=None):
+        manager = manager or self.manager
+        if not manager:
+            raise RuntimeError('this object has not been attached to a manager')
+        manager.save(self)
 
     def _dump(self, manager):
         self.attach(manager)
